SystemOrganization addCategory: #BugFinder!

Object subclass: #Bug
	instanceVariableNames: 'exception message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BugFinder'!

!Bug methodsFor: 'comparing' stamp: 'GonzaloMunoz 12/21/2011 14:37'!
= anObject
	^ (anObject isKindOf: Bug)
		and: [ message = anObject message
		and: [ exception signaler = anObject exception signaler
		and: [ exception messageText = anObject exception messageText ] ] ].! !

!Bug methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/19/2011 18:21'!
exception
	^ exception.! !

!Bug methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/19/2011 18:21'!
exception: anException
	exception := anException.! !

!Bug methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/19/2011 18:20'!
message
	^ message.! !

!Bug methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/19/2011 18:21'!
message: aMessage
	message := aMessage.! !

Object subclass: #BugFinder
	instanceVariableNames: 'generator results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BugFinder'!

!BugFinder class methodsFor: 'as yet unclassified' stamp: 'GonzaloMunoz 12/19/2011 20:34'!
tryUnarySelectors: anInstance iterations: numberOfIterations
	^ self new tryUnarySelectors: anInstance iterations: numberOfIterations.! !

!BugFinder methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/14/2011 22:30'!
generator: aRandom
	generator := aRandom.! !

!BugFinder methodsFor: 'initialization' stamp: 'GonzaloMunoz 12/14/2011 22:28'!
initialize
	generator := Random new.! !

!BugFinder methodsFor: 'results' stamp: 'GonzaloMunoz 12/21/2011 14:38'!
inspectResults
	results inspect.! !

!BugFinder methodsFor: 'comment' stamp: 'GonzaloMunoz 12/21/2011 18:20'!
readMe
	"
	To run random tests on a object:
		BugFinder tryUnarySelectors: anObject iterations: 100.
	
	This returns an instance of BugFinder that stores the results of the tests
	in an OrderedCollection (instance variable 'results'). Each element of this
	collection is a list of bugs found during each iteration.
	
	To see a graph of how many different bugs were found:
		( BugFinder tryUnarySelectors: anObject iterations: 100 ) showGraph.
	
	To inspect the results of the tests:
		( BugFinder tryUnarySelectors: anObject iterations: 100 ) inspectResults.
	"! !

!BugFinder methodsFor: 'accessing' stamp: 'GonzaloMunoz 12/21/2011 14:37'!
results
	^ results.! !

!BugFinder methodsFor: 'results' stamp: 'GonzaloMunoz 12/21/2011 16:57'!
showGraph
	| diag |
	diag := ESDiagramRenderer new.
	(diag lineDiagram)
		y: [ :each | each size ];
		width: 700;
		height: 500;
		valueAxis;
		models: results.
	diag open.! !

!BugFinder methodsFor: 'public' stamp: 'GonzaloMunoz 12/21/2011 14:37'!
tryUnarySelectors: anInstance iterations: numberOfIterations
	| bugsPerIteration exception point caughtBugs bug selector |
	bugsPerIteration := OrderedCollection new.
	caughtBugs := OrderedCollection new.
	numberOfIterations
		timesRepeat: [ 
			selector := (self understandableUnarySelectorsOf: anInstance) atRandom: generator.
			[ anInstance perform: selector ]
				on: Exception
				do: [ :ex | 
					bug := Bug new
						exception: ex;
						message: selector;
						yourself.
					(ex signaler == anInstance or: [ caughtBugs anySatisfy: [ :each | each = bug ] ])
						ifFalse: [ caughtBugs add: bug ] ].
			bugsPerIteration
				add:
					(OrderedCollection new
						addAll: caughtBugs;
						yourself) ].
	results := bugsPerIteration! !

!BugFinder methodsFor: 'util' stamp: 'GonzaloMunoz 12/19/2011 14:26'!
understandableUnarySelectorsOf: anInstance
	^ anInstance class selectors select: [ :s | s isUnary and: [ (s equalsTo: 'initialize') not ] ]! !
